# -*- coding: utf-8 -*-
import os
import copy

import pytest

from boussole.conf.model import DEFAULT_SETTINGS, Settings


def test_001_default(settings, sample_project_settings):
    """conf.Settings: Create a empty Settings object width default values"""
    settings_object = Settings()

    assert settings_object._settings == DEFAULT_SETTINGS

    assert settings_object.TARGET_PATH == DEFAULT_SETTINGS['TARGET_PATH']
    assert settings_object.OUTPUT_STYLES == DEFAULT_SETTINGS['OUTPUT_STYLES']


def test_002_minimal(settings, sample_project_settings):
    """conf.Settings: Very minimalistic settings"""
    minimal_conf = {
        'SOURCES_PATH': '.',
        'TARGET_PATH': './css',
    }

    settings_object = Settings(initial=minimal_conf)

    attempted = copy.deepcopy(DEFAULT_SETTINGS)
    attempted.update(minimal_conf)

    assert settings_object._settings == attempted

    assert settings_object.TARGET_PATH == attempted['TARGET_PATH']
    assert settings_object.OUTPUT_STYLES == attempted['OUTPUT_STYLES']



# -*- coding: utf-8 -*-
import os
import copy
import pytest

from boussole.conf.base_backend import SettingsBackendBase


def test_001(settings, custom_project_settings):
    """conf.base_backend.SettingsBackendBase: Ensure cleaning dont drop anything"""
    backend = SettingsBackendBase(basedir=settings.fixtures_path)

    assert backend.clean(custom_project_settings) == custom_project_settings



# -*- coding: utf-8 -*-
import os
import pytest

# Temporary
#ALL_SOURCES = [
    #os.path.join(settings.sample_path, 'main_syntax.scss'),
    #os.path.join(settings.sample_path, 'main_commented.scss'),
    #os.path.join(settings.sample_path, 'main_basic.scss'),
    #os.path.join(settings.sample_path, 'main_depth_import-1.scss'),
    #os.path.join(settings.sample_path, 'main_depth_import-2.scss'),
    #os.path.join(settings.sample_path, 'main_depth_import-3.scss'),
    #os.path.join(settings.sample_path, 'main_with_subimports.scss'),
    #os.path.join(settings.sample_path, 'main_using_libs.scss'),
    #os.path.join(settings.sample_path, 'main_circular_0.scss'),
    #os.path.join(settings.sample_path, 'main_circular_1.scss'),
    #os.path.join(settings.sample_path, 'main_circular_2.scss'),
    #os.path.join(settings.sample_path, 'main_circular_3.scss'),
    #os.path.join(settings.sample_path, 'main_circular_4.scss'),
    #os.path.join(settings.sample_path, 'main_circular_bridge.scss'),
    #os.path.join(settings.sample_path, 'main_circular_5.scss'),
#]


def test_001(settings, inspector):
    """inspector.ScssInspector: Basic sample in confined space"""
    sourcepath = os.path.join(settings.sample_path, 'main_basic.scss')
    inspector.inspect(sourcepath)

    children = list(inspector.children(sourcepath))
    assert children == [
        os.path.join(settings.sample_path, '_empty.scss'),
        os.path.join(settings.sample_path, '_vendor.scss'),
    ]

    parents = list(inspector.parents(sourcepath))
    assert parents == []



# -*- coding: utf-8 -*-
import os
import pytest


def test_ok_001(settings, parser, resolver):
    """resolver.ImportPathsResolver: Check candidates correct case 1"""
    candidates = resolver.candidate_paths("vendor")

    results = resolver.check_candidate_exists(settings.sample_path, candidates)

    assert results == [os.path.join(settings.sample_path, "_vendor.scss")]


def test_ok_002(settings, parser, resolver):
    """resolver.ImportPathsResolver: Check candidates correct case 2"""
    candidates = resolver.candidate_paths("components/_filename_test_2")

    results = resolver.check_candidate_exists(settings.sample_path, candidates)

    assert results == [os.path.join(settings.sample_path, "components/_filename_test_2.scss")]


def test_ok_003(settings, parser, resolver):
    """resolver.ImportPathsResolver: Check candidates correct case 3"""
    candidates = resolver.candidate_paths("components/filename_test_6.plop.scss")

    results = resolver.check_candidate_exists(settings.sample_path, candidates)

    assert results == [os.path.join(settings.sample_path, "components/_filename_test_6.plop.scss")]


def test_ok_004(settings, parser, resolver):
    """resolver.ImportPathsResolver: Check candidates correct case 4"""
    basepath = os.path.join(settings.sample_path, "components")
    candidates = resolver.candidate_paths("webfont")

    results = resolver.check_candidate_exists(basepath, candidates)

    assert results == [os.path.join(basepath, "_webfont.scss")]


def test_ok_005(settings, parser, resolver):
    """resolver.ImportPathsResolver: Check candidates correct case 5"""
    basepath = os.path.join(settings.sample_path, "components")
    candidates = resolver.candidate_paths("../components/webfont_icons")

    results = resolver.check_candidate_exists(basepath, candidates)

    assert results == [os.path.join(basepath, "../components/_webfont_icons.scss")]


def test_ok_006(settings, parser, resolver):
    """resolver.ImportPathsResolver: Check multiple candidates case 1"""
    candidates = resolver.candidate_paths("components/twin_3")

    results = resolver.check_candidate_exists(settings.sample_path, candidates)

    assert results == [
        os.path.join(settings.sample_path, "components/twin_3.scss"),
        os.path.join(settings.sample_path, "components/_twin_3.scss"),
    ]


def test_ok_007(settings, parser, resolver):
    """resolver.ImportPathsResolver: Check multiple candidates case 2"""
    candidates = resolver.candidate_paths("components/twin_2")

    results = resolver.check_candidate_exists(settings.sample_path, candidates)

    assert results == [
        os.path.join(settings.sample_path, "components/twin_2.scss"),
        os.path.join(settings.sample_path, "components/_twin_2.scss"),
        os.path.join(settings.sample_path, "components/_twin_2.css"),
    ]


def test_wrong_001(settings, parser, resolver):
    """resolver.ImportPathsResolver: Check candidates wrong case 1"""
    candidates = resolver.candidate_paths("dont_exists")

    results = resolver.check_candidate_exists(settings.sample_path, candidates)

    assert results == []


def test_wrong_002(settings, parser, resolver):
    """resolver.ImportPathsResolver: Check candidates wrong case 2"""
    candidates = resolver.candidate_paths("css_filetest.sass")

    results = resolver.check_candidate_exists(settings.sample_path, candidates)

    assert results == []



# -*- coding: utf-8 -*-
import os
import copy
import pytest

from boussole.conf.post_processor import SettingsPostProcessor


def test_001_expand_nothingtodo(settings, sample_project_settings):
    """conf.post_processor.SettingsPostProcessor: Nothing to expand, path is correct"""
    processor = SettingsPostProcessor()

    expand_result = processor._patch_expand_path(sample_project_settings, "DUMMY_NAME", "/foo/bar")

    assert expand_result == "/foo/bar"


def test_002_expand_homedir(settings, sample_project_settings):
    """conf.post_processor.SettingsPostProcessor: Single path expand home user dir"""
    processor = SettingsPostProcessor()

    expand_result = processor._patch_expand_path(sample_project_settings, "DUMMY_NAME", "~/foo")

    assert expand_result == os.path.join(os.environ['HOME'], 'foo')


def test_003_expand_absolute(settings, sample_project_settings):
    """conf.post_processor.SettingsPostProcessor: Single path expand to absolute dir"""
    processor = SettingsPostProcessor()
    processor.projectdir = settings.fixtures_path

    expand_result = processor._patch_expand_path(sample_project_settings,
                                               "DUMMY_NAME",
                                               "tests/data_fixtures/sample_project")

    assert expand_result == os.path.join(settings.fixtures_path,
                                         "tests/data_fixtures/sample_project")


def test_004_expand_wrong(settings, sample_project_settings):
    """conf.post_processor.SettingsPostProcessor: The path does not exists but is expanded
       to absolute from current dir"""
    processor = SettingsPostProcessor()
    processor.projectdir = "/home/user"

    expand_result = processor._patch_expand_path(sample_project_settings,
                                               "DUMMY_NAME", "foo/coco/bar")

    assert expand_result == os.path.join("/home/user", "foo/coco/bar")


def test_005_expand_normpath(settings, sample_project_settings):
    """conf.post_processor.SettingsPostProcessor: Single path expand to absolute dir
       normalized"""
    processor = SettingsPostProcessor()
    processor.projectdir = settings.fixtures_path

    expand_result = processor._patch_expand_path(sample_project_settings,
                                               "DUMMY_NAME",
                                               "tests/data_fixtures/foo/../sample_project")

    assert expand_result == os.path.join(settings.fixtures_path,
                                         "tests/data_fixtures/sample_project")


def test_010_expands(settings, sample_project_settings):
    """conf.post_processor.SettingsPostProcessor: Expand a list of path"""
    processor = SettingsPostProcessor()
    processor.projectdir = settings.fixtures_path

    paths = [
        "/foo/bar",
        "~/foo",
        "tests/data_fixtures/sample_project",
        "tests/data_fixtures/foo/../sample_project",
        "foo/coco/bar",
    ]

    expected = [
        "/foo/bar",
        os.path.join(os.environ['HOME'], 'foo'),
        os.path.join(settings.fixtures_path, "tests/data_fixtures/sample_project"),
        os.path.join(settings.fixtures_path, "tests/data_fixtures/sample_project"),
        os.path.join(settings.fixtures_path, "foo/coco/bar"),
    ]

    expand_result = processor._patch_expand_paths(sample_project_settings, "DUMMY_NAME", paths)

    assert expand_result == expected



"""
Some fixture methods
"""
import os
import pytest

import boussole

from boussole.parser import ScssImportsParser
from boussole.finder import ScssFinder
from boussole.resolver import ImportPathsResolver
from boussole.inspector import ScssInspector
from boussole.compiler import SassCompileHelper
from boussole.project import ProjectStarter


class FixturesSettingsTestMixin(object):
    """Mixin containing some basic settings"""
    def __init__(self):
        # Base fixture datas directory
        self.tests_dir = 'tests'
        self.tests_path = os.path.normpath(
            os.path.join(
                os.path.abspath(os.path.dirname(boussole.__file__)),
                '..',
                self.tests_dir,
            )
        )
        self.fixtures_dir = 'data_fixtures'
        self.fixtures_path = os.path.join(
            self.tests_path,
            self.fixtures_dir
        )

        # Sample project
        self.sample_dir = "sample_project"
        self.sample_path = os.path.join(self.fixtures_path, self.sample_dir)

        # Some sample libraries
        self.lib1_dir = 'library_1'
        self.lib2_dir = 'library_2'
        self.lib1_path = os.path.join(self.fixtures_path, self.lib1_dir)
        self.lib2_path = os.path.join(self.fixtures_path, self.lib2_dir)
        self.libraries_fixture_paths = [
            self.lib1_path,
            self.lib2_path,
        ]


@pytest.fixture(scope='session')
def temp_builds_dir(tmpdir_factory):
    """Prepare a temporary build directory"""
    fn = tmpdir_factory.mktemp('builds')
    return fn


@pytest.fixture(scope="module")
def settings():
    """Initialize and return settings (mostly paths) for fixtures (scope at module level)"""
    return FixturesSettingsTestMixin()


@pytest.fixture(scope="module")
def parser():
    """Initialize and return SCSS parser (scope at module level)"""
    return ScssImportsParser()


@pytest.fixture(scope="module")
def resolver():
    """Initialize and return Path resolver (scope at module level)"""
    return ImportPathsResolver()


@pytest.fixture(scope="function")
def inspector():
    """Initialize and return SCSS inspector (scope at function level)"""
    return ScssInspector()


@pytest.fixture(scope="module")
def finder():
    """Initialize and return SCSS finder (scope at module level)"""
    return ScssFinder()


@pytest.fixture(scope="module")
def compiler():
    """Initialize and return SCSS compile helper (scope at module level)"""
    return SassCompileHelper()


@pytest.fixture(scope="module")
def projectstarter():
    """Initialize and return Project starter (scope at module level)"""
    return ProjectStarter()


@pytest.fixture(scope="module")
def sample_project_settings():
    """Return sample settings dictionnary with expected values (scope at
       module level).

       Warning, this will raise exception from everything involving
       'Backend.clean()' because every paths does not exists"""
    return {
        #'COMPILER_ARGS': [],
        'LIBRARY_PATHS': [
            u'/home/lib1',
            u'/home/lib2',
        ],
        'SOURCES_PATH': u'/home/foo',
        'TARGET_PATH': u'/home/bar',
        'OUTPUT_STYLES': u'nested',
        'SOURCE_COMMENTS': False,
        "EXCLUDES": [],
    }


@pytest.fixture(scope="module")
def custom_project_settings():
    """Return custom settings dictionnary with expected values (scope at
       module level)"""
    fixtures_settings = FixturesSettingsTestMixin()
    return {
        'LIBRARY_PATHS': [
            os.path.join(fixtures_settings.fixtures_path, u'library_1'),
            os.path.join(fixtures_settings.fixtures_path, u'library_2'),
        ],
        'SOURCES_PATH': os.path.join(fixtures_settings.fixtures_path, u'sample_project'),
        'TARGET_PATH': fixtures_settings.fixtures_path,
        'OUTPUT_STYLES': u'expanded',
        'SOURCE_COMMENTS': True,
        "EXCLUDES": [
            "main_error.scss",
            "main_circular_5.scss",
            "main_circular_0.scss",
            "main_circular_4.scss",
            "main_circular_3.scss",
            "main_circular_1.scss",
            "main_circular_2.scss",
            "main_circular_bridge.scss",
            "main_twins_*.scss",
            "*/twin_*.scss"
        ],
    }



# -*- coding: utf-8 -*-
"""
Some utilities and common stuff for tests
"""
import os

from boussole.conf.model import Settings
from boussole.inspector import ScssInspector
from boussole.watcher import SassLibraryEventHandler, SassProjectEventHandler
from boussole.logs import init_logger

class DummyBaseEvent(object):
    """
    Dummy event to pass to almost all handler event methods
    """
    def __init__(self, filepath):
        self.src_path = filepath


class DummyMoveEvent(object):
    """
    Dummy event to pass to handler event 'move' method
    """
    def __init__(self, filepath):
        self.dest_path = filepath


class DummyBaseHandler(object):
    """
    Fake watchdog event handler

    Reproduce some behavior from watchdog event handler to ease tests
    """
    def __init__(self, *args, **kwargs):
        self.patterns = kwargs.pop('patterns')
        self.ignore_patterns = kwargs.pop('ignore_patterns')
        self.ignore_directories = kwargs.pop('ignore_directories')
        self.case_sensitive = kwargs.pop('case_sensitive')
        super(DummyBaseHandler, self).__init__(*args, **kwargs)

    def on_moved(self, event):
        self.on_any_event(event)
        super(DummyBaseHandler, self).on_moved(event)

    def on_created(self, event):
        self.on_any_event(event)
        super(DummyBaseHandler, self).on_created(event)

    def on_modified(self, event):
        self.on_any_event(event)
        super(DummyBaseHandler, self).on_modified(event)

    def on_deleted(self, event):
        self.on_any_event(event)
        super(DummyBaseHandler, self).on_deleted(event)


class UnitTestableLibraryEventHandler(DummyBaseHandler, SassLibraryEventHandler):
    """
    Testable watch event handler for library sources
    """
    pass


class UnitTestableProjectEventHandler(DummyBaseHandler, SassProjectEventHandler):
    """
    Testable watch event handler for project sources
    """
    pass


def join_basedir(basedir):
    """
    Shortcut to join basedir to given path
    """
    def proceed_joining(path):
        return os.path.join(basedir, path)
    return proceed_joining


def start_env(basedir):
    """
    Init all needed stuff for handler testing
    """
    join_basedir_curry = join_basedir(basedir.strpath)

    logger = init_logger('DEBUG', printout=False)

    inspector = ScssInspector()

    minimal_conf = {
        'SOURCES_PATH': basedir.join('sass').strpath,
        'TARGET_PATH': basedir.join('css').strpath,
        'LIBRARY_PATHS': [basedir.join('lib').strpath],
    }
    settings = Settings(initial=minimal_conf)

    watcher_opts = {
        'patterns': ['*.scss'],
        'ignore_patterns': ['*.part'],
        'ignore_directories': False,
        'case_sensitive': True,
    }
    return join_basedir_curry, logger, inspector, settings, watcher_opts


def build_sample_structure(settings_object, basedir):
    """
    Build sample files structure for handler testing
    """
    # Write needed dirs
    os.makedirs(settings_object.SOURCES_PATH)
    os.makedirs(settings_object.TARGET_PATH)
    os.makedirs(os.path.join(settings_object.LIBRARY_PATHS[0], "components"))

    # Write a minimal main SASS source importing partial
    source = "\n".join((
        """/* Main sample */""",
        """@import "toinclude";""",
        """#content{ color: red; }""",
    ))
    with open(basedir.join('sass/main.scss').strpath, 'w') as f:
        f.write(source)

    # Write a main SASS source importing minimal source
    source = "\n".join((
        """/* Main importing sample */""",
        """@import "main";""",
    ))
    with open(basedir.join('sass/main_importing.scss').strpath, 'w') as f:
        f.write(source)

    # Write a main SASS source importing library component and partial source
    source = "\n".join((
        """/* Main importing library */""",
        """@import "toinclude";""",
        """@import "components/buttons";""",
    ))
    with open(basedir.join('sass/main_usinglib.scss').strpath, 'w') as f:
        f.write(source)

    # Write a partial SASS source to include
    source = "\n".join((
        """/* Partial source to include */""",
        """.included-partial{ color: gold !important; }""",
    ))
    with open(basedir.join('sass/_toinclude.scss').strpath, 'w') as f:
        f.write(source)

    # Write a partial SASS source to ignore
    source = "\n".join((
        """/* Partial source to ignore because not included */""",
        """.toignore-partial{ font-weight: bold; }""",
    ))
    with open(basedir.join('sass/_notincluded.scss').strpath, 'w') as f:
        f.write(source)

    # Write a main source within library directory
    source = "\n".join((
        """/* Main source for library */""",
        """@import "components/buttons";""",
    ))
    with open(basedir.join('lib/libmain.scss').strpath, 'w') as f:
        f.write(source)

    # Write a partial source within library directory
    source = "\n".join((
        """/* Buttons component */""",
        """.button{ display: block; border: 1px solid black; padding: 5px; }""",
    ))
    with open(basedir.join('lib/components/_buttons.scss').strpath, 'w') as f:
        f.write(source)



# -*- coding: utf-8 -*-
import os
import click

from boussole.conf.json_backend import SettingsBackendJson
from boussole.exceptions import BoussoleBaseException, SettingsBackendError
from boussole.finder import ScssFinder
from boussole.compiler import SassCompileHelper


@click.command('watch', short_help='Compile SASS project sources to CSS.')
@click.option('--config', default=None, metavar='PATH',
              help='Path to a Boussole config file',
              type=click.Path(exists=True))
@click.pass_context
def compile_command(context, config):
    """
    Compile SASS project sources to CSS
    """
    logger = context.obj['logger']
    logger.info("Building project")

    # Load settings file
    try:
        backend = SettingsBackendJson(basedir=os.getcwd())
        settings = backend.load(filepath=config)
    except SettingsBackendError as e:
        logger.critical(e.message)
        raise click.Abort()

    logger.debug("Project sources directory: {}".format(
                settings.SOURCES_PATH))
    logger.debug("Project destination directory: {}".format(
                settings.TARGET_PATH))
    logger.debug("Exclude patterns: {}".format(
                settings.EXCLUDES))

    # Find all sources with their destination path
    try:
        compilable_files = ScssFinder().mirror_sources(
            settings.SOURCES_PATH,
            targetdir=settings.TARGET_PATH,
            excludes=settings.EXCLUDES
        )
    except BoussoleBaseException as e:
        logger.error(e.message)
        raise click.Abort()

    # Build all compilable stylesheets
    compiler = SassCompileHelper()
    errors = 0
    for src, dst in compilable_files:
        logger.debug("Compile: {}".format(src))

        output_opts = {}
        success, message = compiler.safe_compile(settings, src, dst)

        if success:
            logger.info("Output: {}".format(message), **output_opts)
        else:
            errors += 1
            logger.error(message)

    # Ensure correct exit code if error has occured
    if errors:
        raise click.Abort()



# -*- coding: utf-8 -*-
import os
import json
import pytest

from boussole.exceptions import SettingsInvalidError


def test_success_001(projectstarter, temp_builds_dir):
    """project.init: Testing on default values"""
    basedir = temp_builds_dir.join('projectstarter_init_success_001').strpath
    os.makedirs(basedir)

    results = projectstarter.init(*(
        '.',
        'settings.json',
        'scss',
        'css',
    ), cwd=basedir)

    assert results == {
        "basedir": basedir,
        "config": os.path.join(basedir, "settings.json"),
        "sourcedir": os.path.join(basedir, "scss"),
        "targetdir": os.path.join(basedir, "css"),
    }

    assert os.path.exists(os.path.join(basedir, "settings.json")) == True
    assert os.path.exists(os.path.join(basedir, "scss")) == True
    assert os.path.exists(os.path.join(basedir, "css")) == True

    with open(os.path.join(basedir, "settings.json"), "rb") as fp:
        assert json.load(fp) == {
            'SOURCES_PATH': 'scss',
            'TARGET_PATH': 'css',
            "LIBRARY_PATHS": [],
            "OUTPUT_STYLES": "nested",
            "SOURCE_COMMENTS": False,
            "EXCLUDES": []
        }


def test_error_001(projectstarter, temp_builds_dir):
    """project.init: Raised exception caused by duplicate paths"""
    basedir = temp_builds_dir.join('projectstarter_init_error_001').strpath
    os.makedirs(basedir)

    with pytest.raises(SettingsInvalidError):
        results = projectstarter.init(*(
            '.',
            'settings.json',
            'css',
            'css',
        ), cwd=basedir)



# -*- coding: utf-8 -*-
import pytest


def test_remove_001(settings, parser):
    """parser.ScssImportsParser: removing singleline comment case 1"""
    assert parser.remove_comments("""// foo""") == ""


def test_remove_002(settings, parser):
    """parser.ScssImportsParser: removing singleline comment case 2"""
    assert parser.remove_comments("""//foo
        """).strip() == ""


def test_remove_003(settings, parser):
    """parser.ScssImportsParser: removing singleline comment case 3"""
    assert parser.remove_comments("""
        //foo
    """).strip() == ""


def test_remove_004(settings, parser):
    """parser.ScssImportsParser: removing singleline comment case 4"""
    assert parser.remove_comments("""$foo: true;
// foo
$bar: false;
""").strip() == """$foo: true;\n$bar: false;"""


def test_remove_005(settings, parser):
    """parser.ScssImportsParser: removing singleline comment case 5"""
    results = parser.remove_comments("""@import "vendor"; //foo""").strip()
    assert results == """@import "vendor";"""


def test_remove_006(settings, parser):
    """parser.ScssImportsParser: removing multiline comment case 1"""
    assert parser.remove_comments("""/* foo */""") == ""


def test_remove_007(settings, parser):
    """parser.ScssImportsParser: removing multiline comment case 2"""
    assert parser.remove_comments("""
        /*
            * foo
            */""").strip() == ""


def test_remove_008(settings, parser):
    """parser.ScssImportsParser: removing multiline comment case 3"""
    assert parser.remove_comments("""
        /*
            * foo
            */
            $bar: true;""").strip() == "$bar: true;"


def test_remove_009(settings, parser):
    """parser.ScssImportsParser: removing singleline and multiline comments"""
    assert parser.remove_comments("""//Start
/*
 * Pika
 */
$foo: true;
// Boo
$bar: false;
// End""").strip() == "$foo: true;\n$bar: false;"


def test_remove_010(settings, parser):
    """parser.ScssImportsParser: trouble with // usage that are not comment"""
    assert parser.remove_comments("""@import url("http://foo.bar/dummy");""") == """@import url("http://foo.bar/dummy");"""


def test_remove_011(settings, parser):
    """parser.ScssImportsParser: trouble with // usage that are not comment"""
    assert parser.remove_comments("""@import url("http://foo.bar/dummy"); // This is a comment""") == """@import url("http://foo.bar/dummy"); """



# -*- coding: utf-8 -*-
import pytest

from boussole.exceptions import InvalidImportRule

def test_ok_001(settings, parser):
    """parser.ScssImportsParser: Single quote"""
    assert parser.strip_quotes("'foo'") == "foo"


def test_ok_002(settings, parser):
    """parser.ScssImportsParser: Double quotes"""
    assert parser.strip_quotes('"foo"') == "foo"


def test_ok_003(settings, parser):
    """parser.ScssImportsParser: Without quotes"""
    assert parser.strip_quotes("foo") == "foo"


def test_error_001(settings, parser):
    """parser.ScssImportsParser: Error, quote starting but not ended"""
    with pytest.raises(InvalidImportRule):
        parser.strip_quotes("'foo")

    with pytest.raises(InvalidImportRule):
        parser.strip_quotes('"foo')


def test_error_002(settings, parser):
    """parser.ScssImportsParser: Error, quote ending but not started"""
    with pytest.raises(InvalidImportRule):
        parser.strip_quotes('foo"')

    with pytest.raises(InvalidImportRule):
        parser.strip_quotes("foo'")



# -*- coding: utf-8 -*-
import os
import pytest


def test_001(settings, finder):
    """finder.ScssFinder: Simple filename"""
    partial = finder.is_partial("foo.scss")
    assert partial == False


def test_002(settings, finder):
    """finder.ScssFinder: Simple partial filename"""
    partial = finder.is_partial("_foo.scss")
    assert partial == True


def test_003(settings, finder):
    """finder.ScssFinder: Simple relative filename"""
    partial = finder.is_partial("bar/foo.scss")
    assert partial == False


def test_004(settings, finder):
    """finder.ScssFinder: Simple relative partial filename"""
    partial = finder.is_partial("bar/_foo.scss")
    assert partial == True


def test_005(settings, finder):
    """finder.ScssFinder: Simple relative filename again"""
    partial = finder.is_partial("bar/plop/foo.scss")
    assert partial == False


def test_006(settings, finder):
    """finder.ScssFinder: Simple relative partial filename again"""
    partial = finder.is_partial("bar/plop/_foo.scss")
    assert partial == True


def test_007(settings, finder):
    """finder.ScssFinder: Simple absolute filename"""
    partial = finder.is_partial("/home/bar/foo.scss")
    assert partial == False


def test_008(settings, finder):
    """finder.ScssFinder: Simple absolute partial filename"""
    partial = finder.is_partial("/home/bar/_foo.scss")
    assert partial == True



# -*- coding: utf-8 -*-
import os
import copy
import pytest

from boussole.conf.post_processor import SettingsPostProcessor


class SettingsPostProcessorTest(SettingsPostProcessor):
    """
    Inherit to add some dummy methods and its own manifest for testing
    processor itself
    """
    settings_manifesto = {
        'field1': {
            'default': [],
            'postprocess': (
                '_dummy_start',
            ),
        },
        'field2': {
            'default': None,
            'postprocess': (
                '_dummy_start',
                '_dummy_end1',
                '_dummy_end2',
            ),
        },
        'field3': {
            'default': [],
            'postprocess': (
                '_dummy_end2',
                '_dummy_end1',
            ),
        },
        'field4': {
            'default': [],
            'postprocess': [],
        },
    }


    def _dummy_start(self, settings, name, value):
        """
        Add 'Hello' at the start of value
        """
        return "Hello {}".format(value)

    def _dummy_end1(self, settings, name, value):
        """
        Add 'world' at the end of value
        """
        return "{} world".format(value)

    def _dummy_end2(self, settings, name, value):
        """
        Add '!' at the end of value
        """
        return "{} !".format(value)


def test_001_dummy_methods(settings, sample_project_settings):
    """conf.post_processor.SettingsPostProcessor: Just directly test dummy methods"""
    processor = SettingsPostProcessorTest()

    expand_result = processor._dummy_start(sample_project_settings, "DUMMY_NAME", "goodbye")

    assert expand_result == "Hello goodbye"

    expand_result = processor._dummy_end1(sample_project_settings, "DUMMY_NAME", "I rule the")

    assert expand_result == "I rule the world"

    expand_result = processor._dummy_end2(sample_project_settings, "DUMMY_NAME", "Hi")

    assert expand_result == "Hi !"


def test_002_chain_basic(settings, sample_project_settings):
    """conf.post_processor.SettingsPostProcessor: Testing methods chaining on single rule"""
    processor = SettingsPostProcessorTest()

    expand_result = processor.post_process({
        "field1": "world"
    })

    assert expand_result == {"field1": "Hello world"}


def test_003_chain_more(settings, sample_project_settings):
    """conf.post_processor.SettingsPostProcessor: Testing methods chaining with many rules"""
    processor = SettingsPostProcessorTest()

    expand_result = processor.post_process({
        "field2": "my"
    })

    assert expand_result == {"field2": "Hello my world !"}


def test_004_chain_order(settings, sample_project_settings):
    """conf.post_processor.SettingsPostProcessor: Testing methods chaining order is respected"""
    processor = SettingsPostProcessorTest()

    expand_result = processor.post_process({
        "field3": "foo"
    })

    assert expand_result == {"field3": "foo ! world"}


def test_005_chain_empty(settings, sample_project_settings):
    """conf.post_processor.SettingsPostProcessor: Testing methods chaining with no rules"""
    processor = SettingsPostProcessorTest()

    expand_result = processor.post_process({
        "field4": "foo"
    })

    assert expand_result == {"field4": "foo"}


def test_006_chain_complete(settings, sample_project_settings):
    """conf.post_processor.SettingsPostProcessor: Testing methods chaining on all fields"""
    processor = SettingsPostProcessorTest()

    expand_result = processor.post_process({
        "field1": "world",
        "field2": "my",
        "field3": "foo",
        "field4": "foo",
    })

    assert expand_result == {
        "field1": "Hello world",
        "field2": "Hello my world !",
        "field3": "foo ! world",
        "field4": "foo",
    }



# -*- coding: utf-8 -*-
import os
import json
import pytest

from boussole.exceptions import SettingsInvalidError


def test_ok_001(projectstarter):
    """project.valid_paths: correct paths"""

    assert projectstarter.valid_paths(*(
        '/home/foo',
        '/home/bar',
    )) == True


def test_ok_002(projectstarter):
    """project.valid_paths: correct paths"""

    assert projectstarter.valid_paths(*(
        '/home',
        '/home/bar',
        '/var/lib',
    )) == True


def test_ok_003(projectstarter):
    """project.valid_paths: correct paths"""

    assert projectstarter.valid_paths(*(
        '/home',
        '/home/bar',
        '/home/bar.gif',
        '/var/lib',
    )) == True


def test_wrong_001(projectstarter):
    """project.valid_paths: two identical paths"""

    with pytest.raises(SettingsInvalidError):
        projectstarter.valid_paths(*(
            '/home/foo',
            '/home/bar',
            '/home/foo',
        ))


def test_wrong_002(projectstarter):
    """project.valid_paths: three identical paths"""

    with pytest.raises(SettingsInvalidError):
        projectstarter.valid_paths(*(
            '/home/foo',
            '/home/bar',
            '/home/foo',
            '/home/foo',
        ))


def test_wrong_003(projectstarter):
    """project.valid_paths: multiple identical paths"""

    with pytest.raises(SettingsInvalidError):
        projectstarter.valid_paths(*(
            '/home/foo',
            '/home/bar',
            '/home/foo',
            '/home/bar',
        ))


def test_wrong_004(projectstarter):
    """project.valid_paths: multiple identical paths"""

    with pytest.raises(SettingsInvalidError):
        projectstarter.valid_paths(*(
            '/home/foo',
            '/home/bar',
            '/home/bar',
            '/home/foo',
            '/var/lib',
            '/var/lib',
        ))



"""
Project management
==================
"""
import os
import json

from boussole.exceptions import SettingsInvalidError


class ProjectStarter(object):
    """
    Provide methods to create a new SASS Project
    """
    def valid_paths(self, *args):
        """
        Validate that given paths are not the same.

        Args:
            (string): Path to validate.

        Raises:
            boussole.exceptions.SettingsInvalidError: If there is more than one
                occurence of the same path.

        Returns:
            bool: ``True`` if paths are validated.
        """
        for i, path in enumerate(args, start=0):
            cp = list(args)
            current = cp.pop(i)
            if current in cp:
                raise SettingsInvalidError("Multiple occurences finded for "
                                           "path: {}".format(current))

        return True

    def expand(self, basedir, config, sourcedir, targetdir, cwd):
        """
        Validate that given paths are not the same.

        Args:
            basedir (string): Project base directory used to prepend relative
                paths. If empty or equal to '.', it will be filled with current
                directory path.
            config (string): Settings file path.
            sourcedir (string): Source directory path.
            targetdir (string): Compiled files target directory path.
            cwd (string): Current directory path to prepend base dir if empty.

        Returns:
            tuple: Expanded arguments in the same order
        """

        # Expand home directory if any
        expanded_basedir = os.path.expanduser(basedir)
        expanded_config = os.path.expanduser(config)
        expanded_sourcedir = os.path.expanduser(sourcedir)
        expanded_targetdir = os.path.expanduser(targetdir)

        # If not absolute, base dir is prepended with current directory
        if not os.path.isabs(expanded_basedir):
            expanded_basedir = os.path.join(cwd, expanded_basedir)
        # Prepend paths with base dir if they are not allready absolute
        if not os.path.isabs(expanded_config):
            expanded_config = os.path.join(expanded_basedir,
                                           expanded_config)
        if not os.path.isabs(expanded_sourcedir):
            expanded_sourcedir = os.path.join(expanded_basedir,
                                              expanded_sourcedir)
        if not os.path.isabs(expanded_targetdir):
            expanded_targetdir = os.path.join(expanded_basedir,
                                              expanded_targetdir)

        # Normalize paths
        expanded_basedir = os.path.normpath(expanded_basedir)
        expanded_config = os.path.normpath(expanded_config)
        expanded_sourcedir = os.path.normpath(expanded_sourcedir)
        expanded_targetdir = os.path.normpath(expanded_targetdir)

        return (expanded_basedir, expanded_config, expanded_sourcedir,
                expanded_targetdir)

    def commit(self, sourcedir, targetdir, abs_config, abs_sourcedir,
               abs_targetdir):
        """
        Commit project structure and configuration file

        Args:
            sourcedir (string): Source directory path.
            targetdir (string): Compiled files target directory path.
            abs_config (string): Configuration file absolute path.
            abs_sourcedir (string): ``sourcedir`` expanded as absolute path.
            abs_targetdir (string): ``targetdir`` expanded as absolute path.
        """
        config_path, config_filename = os.path.split(abs_config)

        if not os.path.exists(config_path):
            os.makedirs(config_path)
        if not os.path.exists(abs_sourcedir):
            os.makedirs(abs_sourcedir)
        if not os.path.exists(abs_targetdir):
            os.makedirs(abs_targetdir)

        # Create settings with given paths
        with open(abs_config, "w") as fp:
            json.dump({
                'SOURCES_PATH': sourcedir,
                'TARGET_PATH': targetdir,
                "LIBRARY_PATHS": [],
                "OUTPUT_STYLES": "nested",
                "SOURCE_COMMENTS": False,
                "EXCLUDES": []
            }, fp, indent=4)

    def init(self, basedir, config, sourcedir, targetdir, cwd='', commit=True):
        """
        Init project structure and configuration from given arguments

        Args:
            basedir (string): Project base directory used to prepend relative
                paths. If empty or equal to '.', it will be filled with current
                directory path.
            config (string): Settings file path.
            sourcedir (string): Source directory path.
            targetdir (string): Compiled files target directory path.

        Keyword Arguments:
            cwd (string): Current directory path to prepend base dir if empty.
            commit (bool): If ``False``, directory structure and settings file
                won't be created.

        Returns:
            dict: A dict containing expanded given paths.
        """
        if not basedir:
            basedir = '.'

        # Expand home directory if any
        abs_basedir, abs_config, abs_sourcedir, abs_targetdir = self.expand(
            basedir, config,
            sourcedir, targetdir,
            cwd
        )

        # Valid every paths are ok
        self.valid_paths(abs_config, abs_sourcedir, abs_targetdir)

        # Create required directory structure
        if commit:
            self.commit(sourcedir, targetdir, abs_config, abs_sourcedir,
                        abs_targetdir)

        return {
            'basedir': abs_basedir,
            'config': abs_config,
            'sourcedir': abs_sourcedir,
            'targetdir': abs_targetdir,
        }



# -*- coding: utf-8 -*-
import os
import pytest


def test_relative_001(settings, finder):
    """finder.ScssFinder: Mirror all compilable sources in relative mode"""
    assert finder.mirror_sources(os.path.join(settings.sample_path, 'components')) == [
        ('twin_3.scss', 'twin_3.css'),
        ('twin_2.scss', 'twin_2.css'),
    ]


def test_absolute_002(settings, finder):
    """finder.ScssFinder: Mirror all compilable sources in absolute mode"""
    assert finder.mirror_sources(os.path.join(settings.sample_path, 'components'), targetdir=settings.tests_path) == [
        (os.path.join(settings.sample_path, 'components', 'twin_3.scss'), os.path.join(settings.tests_path, 'twin_3.css')),
        (os.path.join(settings.sample_path, 'components', 'twin_2.scss'), os.path.join(settings.tests_path, 'twin_2.css')),
    ]


def test_relative_recursive_003(settings, finder):
    """finder.ScssFinder: Mirror all compilable sources in relative mode"""
    assert finder.mirror_sources(settings.sample_path) == [
        ('main_syntax.scss', 'main_syntax.css'),
        ('main_error.scss', 'main_error.css'),
        ('main_circular_5.scss', 'main_circular_5.css'),
        ('main_depth_import-2.scss', 'main_depth_import-2.css'),
        ('main_twins_2.scss', 'main_twins_2.css'),
        ('main_twins_1.scss', 'main_twins_1.css'),
        ('main_circular_0.scss', 'main_circular_0.css'),
        ('main_circular_4.scss', 'main_circular_4.css'),
        ('main_circular_3.scss', 'main_circular_3.css'),
        ('main_twins_3.scss', 'main_twins_3.css'),
        ('main_using_libs.scss', 'main_using_libs.css'),
        ('main_depth_import-3.scss', 'main_depth_import-3.css'),
        ('main_circular_1.scss', 'main_circular_1.css'),
        ('main_depth_import-1.scss', 'main_depth_import-1.css'),
        ('main_commented.scss', 'main_commented.css'),
        ('main_with_subimports.scss', 'main_with_subimports.css'),
        ('main_circular_2.scss', 'main_circular_2.css'),
        ('main_circular_bridge.scss', 'main_circular_bridge.css'),
        ('main_basic.scss', 'main_basic.css'),
        ('components/twin_3.scss', 'components/twin_3.css'),
        ('components/twin_2.scss', 'components/twin_2.css'),
    ]


def test_absolute_recursive_004(settings, finder):
    """finder.ScssFinder: Mirror all compilable sources in absolute mode"""
    # To avoid to write every os path join, i'm so lazy..
    absolutize = lambda x,y: (os.path.join(settings.sample_path, x), os.path.join(settings.tests_path, y))

    assert finder.mirror_sources(settings.sample_path, targetdir=settings.tests_path) == [
        absolutize('main_syntax.scss', 'main_syntax.css'),
        absolutize('main_error.scss', 'main_error.css'),
        absolutize('main_circular_5.scss', 'main_circular_5.css'),
        absolutize('main_depth_import-2.scss', 'main_depth_import-2.css'),
        absolutize('main_twins_2.scss', 'main_twins_2.css'),
        absolutize('main_twins_1.scss', 'main_twins_1.css'),
        absolutize('main_circular_0.scss', 'main_circular_0.css'),
        absolutize('main_circular_4.scss', 'main_circular_4.css'),
        absolutize('main_circular_3.scss', 'main_circular_3.css'),
        absolutize('main_twins_3.scss', 'main_twins_3.css'),
        absolutize('main_using_libs.scss', 'main_using_libs.css'),
        absolutize('main_depth_import-3.scss', 'main_depth_import-3.css'),
        absolutize('main_circular_1.scss', 'main_circular_1.css'),
        absolutize('main_depth_import-1.scss', 'main_depth_import-1.css'),
        absolutize('main_commented.scss', 'main_commented.css'),
        absolutize('main_with_subimports.scss', 'main_with_subimports.css'),
        absolutize('main_circular_2.scss', 'main_circular_2.css'),
        absolutize('main_circular_bridge.scss', 'main_circular_bridge.css'),
        absolutize('main_basic.scss', 'main_basic.css'),
        absolutize('components/twin_3.scss', 'components/twin_3.css'),
        absolutize('components/twin_2.scss', 'components/twin_2.css'),
    ]


def test_relative_excludes_004(settings, finder):
    """finder.ScssFinder: Mirror allowed compilable sources in relative mode"""
    excludes = [
        'main_error.scss',
        'main_circular_5.scss',
        'main_circular_0.scss',
        'main_circular_4.scss',
        'main_circular_3.scss',
        'main_circular_1.scss',
        'main_circular_2.scss',
        'main_circular_bridge.scss',
        "main_twins_*.scss",
        '*/twin_*.scss',
    ]

    assert finder.mirror_sources(settings.sample_path, excludes=excludes) == [
        ('main_syntax.scss', 'main_syntax.css'),
        ('main_depth_import-2.scss', 'main_depth_import-2.css'),
        ('main_using_libs.scss', 'main_using_libs.css'),
        ('main_depth_import-3.scss', 'main_depth_import-3.css'),
        ('main_depth_import-1.scss', 'main_depth_import-1.css'),
        ('main_commented.scss', 'main_commented.css'),
        ('main_with_subimports.scss', 'main_with_subimports.css'),
        ('main_basic.scss', 'main_basic.css'),
    ]



# -*- coding: utf-8 -*-
import os
import pytest

from boussole.conf.base_backend import SettingsBackendBase


def test_ok_001(settings):
    """conf.base_backend.SettingsBackendBase: Open given filepath"""
    backend = SettingsBackendBase(basedir=settings.fixtures_path)

    path, filename = backend.parse_filepath()
    filepath = backend.check_filepath(path, filename)

    assert backend.open(filepath) == """Fake settings file as SettingsBackendBase dont implement a full usable interface."""



# -*- coding: utf-8 -*-
import os
import pytest


def test_comment_001(settings, parser):
    """parser.ScssImportsParser: commented import 1 (buggy behavior)"""
    result = parser.parse("""//@import "compass/css3";""")
    assert result == []


def test_comment_002(settings, parser):
    """parser.ScssImportsParser: commented import 2 (buggy behavior)"""
    result = parser.parse("""//      @import "compass/css3";""")
    assert result == []


def test_comment_003(settings, parser):
    """parser.ScssImportsParser: commented import 3 (buggy behavior)"""
    result = parser.parse("""/*
        @import "compass/css3";
        */""")
    assert result == []


def test_sample_001(settings, parser):
    """parser.ScssImportsParser: complete file"""
    with open(os.path.join(settings.sample_path, 'main_syntax.scss')) as fp:
        result = parser.parse(fp.read())
    assert result == [
        'vendor', 'utils/mixins', 'sass_filetest', 'css_filetest', '_empty',
        'components/filename_test_1', 'components/_filename_test_2',
        'components/filename_test_3.scss',
        'components/_filename_test_4.scss',
        'components/filename_test_5.plop',
        'components/filename_test_6.plop.scss',
        'components/../empty',
    ]


def test_empty_001(settings, parser):
    """parser.ScssImportsParser: empty file"""
    with open(os.path.join(settings.sample_path, '_empty.scss')) as fp:
        result = parser.parse(fp.read())
    assert result == []


def test_noimport_001(settings, parser):
    """parser.ScssImportsParser: no import rules"""
    with open(os.path.join(settings.sample_path, '_vendor.scss')) as fp:
        result = parser.parse(fp.read())
    assert result == []



# -*- coding: utf-8 -*-
import os
import click

from boussole.exceptions import SettingsInvalidError
from boussole.project import ProjectStarter


@click.command('startproject', short_help="Create a new SASS project.")
@click.option('--basedir', metavar='PATH',
              prompt="Project base directory",
              help=("Base directory where settings filename and project "
                    "structure will be created."),
              default=".")
@click.option('--config', metavar='PATH',
              prompt="Settings file name",
              help="Settings file name",
              default="settings.json")
@click.option('--sourcedir', metavar='PATH',
              prompt="Sources directory",
              help="Directory (within base dir) for your SASS sources.",
              default="scss")
@click.option('--targetdir', metavar='PATH',
              prompt="Target directory",
              help="Directory (within base dir) where to write compiled "
                   "files.",
              default="css")
@click.pass_context
def startproject_command(context, basedir, config, sourcedir, targetdir):
    """
    Create a new SASS project

    This will prompt you to define your project configuration in a settings
    file then create needed directory structure.

    Arguments "basedir", "config", "sourcedir", "targetdir" can not be empty.
    """
    logger = context.obj['logger']

    try:
        results = ProjectStarter().init(*(
            basedir,
            config,
            sourcedir,
            targetdir,
        ), cwd=os.getcwd())
    except SettingsInvalidError as e:
        logger.critical(e.message)
        raise click.Abort()
    else:
        logger.info("Project directory structure and configuration file have "
                    "been created.")

        logger.info("Now you should start to create some SASS sources into "
                    "'{}', then compile them "
                    "using:".format(results['sourcedir']))

        logger.info("    boussole compile "
                    "--config={}".format(results['config']))



# -*- coding: utf-8 -*-
import os
import pytest

from boussole.exceptions import SettingsBackendError
from boussole.conf.base_backend import SettingsBackendBase


def test_ok_001(settings):
    """conf.base_backendSettingsBackendBase: Filepath check case 1"""
    backend = SettingsBackendBase()

    result = backend.check_filepath(settings.fixtures_path, filename=SettingsBackendBase._default_filename)

    assert result == os.path.join(settings.fixtures_path, SettingsBackendBase._default_filename)


def test_ok_002(settings):
    """conf.base_backendSettingsBackendBase: Filepath check case 2"""
    backend = SettingsBackendBase()

    result = backend.check_filepath(settings.sample_path, filename="dummy")

    assert result == os.path.join(settings.sample_path, "dummy")


def test_error_001(settings):
    """conf.base_backendSettingsBackendBase: Filepath check error case 1 (dont exist)"""
    backend = SettingsBackendBase()

    with pytest.raises(SettingsBackendError):
        backend.check_filepath(settings.fixtures_path, filename="dontexists")


def test_error_002(settings):
    """conf.base_backendSettingsBackendBase: Filepath check error case 2 (filename is a dir)"""
    backend = SettingsBackendBase()

    with pytest.raises(SettingsBackendError):
        backend.check_filepath(settings.sample_path, filename="components")



# -*- coding: utf-8 -*-
import os
import copy
import pytest

from boussole.conf.model import Settings

def test_001_basic(settings, sample_project_settings):
    """conf.Settings: Ensure cleans don't drop available settings"""
    settings_object = Settings()

    fake_settings = copy.deepcopy(sample_project_settings)

    assert settings_object.clean(fake_settings) == sample_project_settings

def test_002_custom(settings, sample_project_settings):
    """conf.Settings: Filter unavailable settings"""
    settings_object = Settings()

    fake_settings = copy.deepcopy(sample_project_settings)

    # Add some dummy settings to remove
    fake_settings.update({
        'foo': True,
        'bar': 42,
        'plonk': [],
    })

    assert settings_object.clean(fake_settings) == sample_project_settings



# -*- coding: utf-8 -*-
import os
import json
import pytest


def test_001(projectstarter):
    """project.expand: basic"""

    results = projectstarter.expand(*[
        '.',
        'settings.json',
        'scss',
        'css',
        '/home/foo',
    ])

    assert results == (
        '/home/foo',
        '/home/foo/settings.json',
        '/home/foo/scss',
        '/home/foo/css',
    )


def test_002(projectstarter):
    """project.expand: having absolute path for basedir"""

    results = projectstarter.expand(*[
        '/home/foo',
        'settings.json',
        'scss',
        'css',
        '/home/bar',
    ])

    assert results == (
        '/home/foo',
        '/home/foo/settings.json',
        '/home/foo/scss',
        '/home/foo/css',
    )


def test_003(projectstarter):
    """project.expand: absolute basedir but unused because other paths are absolute also"""

    results = projectstarter.expand(*[
        '/home/bar',
        '/home/foo/settings.json',
        '/home/foo/scss',
        '/home/foo/css',
        '/home/meuh',
    ])

    assert results == (
        '/home/bar',
        '/home/foo/settings.json',
        '/home/foo/scss',
        '/home/foo/css',
    )


def test_004(projectstarter):
    """project.expand: using non basic relative paths"""

    results = projectstarter.expand(*[
        '/home/foo',
        "settings.json",
        "sass/scss",
        "../project/static/css",
        '/home/bar',
    ])

    assert results == (
        '/home/foo',
        '/home/foo/settings.json',
        '/home/foo/sass/scss',
        '/home/project/static/css',
    )


def test_004(projectstarter):
    """project.expand: mixin of everything before"""

    results = projectstarter.expand(*[
        '/home/foo',
        "/home/bar/settings.json",
        "sass/scss",
        "../project/static/css",
        '/home/meuh',
    ])

    assert results == (
        '/home/foo',
        '/home/bar/settings.json',
        '/home/foo/sass/scss',
        '/home/project/static/css',
    )



# -*- coding: utf-8 -*-
import os
import pytest

from boussole.exceptions import SettingsBackendError
from boussole.conf.json_backend import SettingsBackendJson


def test_ok_001(settings, sample_project_settings):
    """conf.json_backend.SettingsBackendJson: JSON content parsing"""
    backend = SettingsBackendJson(basedir=settings.fixtures_path)

    path, filename = backend.parse_filepath()
    filepath = backend.check_filepath(path, filename)

    content = backend.open(filepath)

    assert backend.parse(filepath, content) == sample_project_settings


def test_error_001(settings, sample_project_settings):
    """conf.json_backend.SettingsBackendJson: JSON content parsing error"""
    backend = SettingsBackendJson(basedir=settings.fixtures_path)

    path, filename = backend.parse_filepath(filepath="settings.txt")
    filepath = backend.check_filepath(path, filename)

    content = backend.open(filepath)

    with pytest.raises(SettingsBackendError):
        backend.parse(filepath, content)



# -*- coding: utf-8 -*-
import os
import logging
import pytest

from boussole.exceptions import UnresolvablePath

from utils import (DummyBaseEvent, DummyMoveEvent, DummyBaseHandler,
                   UnitTestableLibraryEventHandler,
                   UnitTestableProjectEventHandler, start_env,
                   build_sample_structure)


def test_index_001(settings, caplog, temp_builds_dir):
    """watcher.SassProjectEventHandler: UnresolvablePath on index from 'on_any_event'"""
    basedir = temp_builds_dir.join('watcher_fails_001')

    bdir, logger, inspector, settings_object, watcher_opts = start_env(basedir)

    build_sample_structure(settings_object, basedir)

    # Init handler
    project_handler = UnitTestableProjectEventHandler(
        settings_object,
        logger,
        inspector,
        **watcher_opts
    )

    #with pytest.raises(UnresolvablePath):
    # First indexing is a success
    project_handler.on_any_event(object())

    # Write some error file
    source = "\n".join((
        """/* New main source */""",
        """@import "idontexist";""",
    ))
    with open(basedir.join('sass/error_main.scss').strpath, 'w') as f:
        f.write(source)

    # Second indexing raise exception but catched
    project_handler.on_any_event(object())

    # Error from core API is catched
    assert project_handler._event_error == True
    # Logged error
    assert caplog.record_tuples == [
        (
            'boussole',
            40,
            "Imported path 'idontexist' does not exist in '{}'".format(basedir.join('sass').strpath)
        )
    ]


def test_deleted_041(settings, caplog, temp_builds_dir):
    """watcher.SassProjectEventHandler: UnresolvablePath on 'Deleted' event for a partial
       source included by other files"""
    basedir = temp_builds_dir.join('watcher_fails_041')

    bdir, logger, inspector, settings_object, watcher_opts = start_env(basedir)

    build_sample_structure(settings_object, basedir)

    # Init handler
    project_handler = UnitTestableProjectEventHandler(
        settings_object,
        logger,
        inspector,
        **watcher_opts
    )

    os.remove(bdir('sass/_toinclude.scss'))

    # Error is catched
    project_handler.on_deleted(DummyBaseEvent(bdir('sass/_toinclude.scss')))

    # Error from core API is catched
    assert project_handler._event_error == True
    # Logged error
    assert caplog.record_tuples == [
        (
            'boussole',
            40,
            "Imported path 'toinclude' does not exist in '{}'".format(basedir.join('sass').strpath)
        )
    ]



# -*- coding: utf-8 -*-
import os
import pytest

def test_001(settings, inspector):
    """inspector.ScssInspector: Sample doing depth imports"""
    sources = [
        os.path.join(settings.sample_path, 'main_depth_import-1.scss'),
        os.path.join(settings.sample_path, 'main_depth_import-2.scss'),
        os.path.join(settings.sample_path, 'main_depth_import-3.scss'),
    ]
    inspector.inspect(*sources)

    # First depth level
    children = list(inspector.children(os.path.join(settings.sample_path, 'main_depth_import-1.scss')))
    assert children == [
        os.path.join(settings.sample_path, "_empty.scss"),
        os.path.join(settings.sample_path, "_vendor.scss"),
        os.path.join(settings.sample_path, "main_basic.scss"),
    ]

    parents = list(inspector.parents(os.path.join(settings.sample_path, 'main_depth_import-1.scss')))
    assert parents == [
        os.path.join(settings.sample_path, "main_depth_import-2.scss"),
        os.path.join(settings.sample_path, "main_depth_import-3.scss"),
    ]

    # Second depth level
    children = list(inspector.children(os.path.join(settings.sample_path, 'main_depth_import-2.scss')))
    assert children == [
        os.path.join(settings.sample_path, "main_depth_import-1.scss"),
        os.path.join(settings.sample_path, "_empty.scss"),
        os.path.join(settings.sample_path, "_vendor.scss"),
        os.path.join(settings.sample_path, "main_basic.scss"),
    ]

    parents = list(inspector.parents(os.path.join(settings.sample_path, 'main_depth_import-2.scss')))
    assert parents == [
        os.path.join(settings.sample_path, "main_depth_import-3.scss"),
    ]

    # Third depth level
    children = list(inspector.children(os.path.join(settings.sample_path, 'main_depth_import-3.scss')))
    assert children == [
        os.path.join(settings.sample_path, "main_depth_import-1.scss"),
        os.path.join(settings.sample_path, "main_depth_import-2.scss"),
        os.path.join(settings.sample_path, "_vendor.scss"),
        os.path.join(settings.sample_path, "main_basic.scss"),
        os.path.join(settings.sample_path, "_empty.scss"),
    ]

    parents = list(inspector.parents(os.path.join(settings.sample_path, 'main_depth_import-3.scss')))
    assert parents == []



from setuptools import setup, find_packages

setup(
    name='boussole',
    version=__import__('boussole').__version__,
    description=__import__('boussole').__doc__,
    long_description=open('README.rst').read(),
    author='David Thenon',
    author_email='sveetch@gmail.com',
    url='https://github.com/sveetch/boussole',
    license='MIT',
    packages=find_packages(),
    classifiers=[
        'Programming Language :: Python',
        "Programming Language :: Python :: 2.7",
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    install_requires=[
        'click==5.1',
        'pathtools==0.1.2',
        'watchdog==0.8.3',
        'libsass>=0.11.0',
        'colorama',
        'colorlog',
    ],
    tests_require=[
        'pytest',
        'pytest-catchlog',
    ],
    entry_points={
        'console_scripts': [
            'boussole = boussole.cli.console_script:cli_frontend',
        ]
    },
    include_package_data=True,
    zip_safe=False
)


# -*- coding: utf-8 -*-
import os
import json

import pytest

import click
from click.testing import CliRunner

from boussole.cli.console_script import cli_frontend


def test_001(settings, caplog):
    """cli.startproject: Basic"""
    runner = CliRunner()

    # Temporary isolated current dir
    with runner.isolated_filesystem():
        test_cwd = os.getcwd()

        sourcedir = os.path.join(test_cwd, "scss")
        targetdir = os.path.join(test_cwd, "css")
        config_filepath = os.path.join(test_cwd, "settings.json")

        opts = [
            'startproject',
            '--basedir={}'.format(test_cwd),
            '--config={}'.format("settings.json"),
            '--sourcedir={}'.format("scss"),
            '--targetdir={}'.format("css"),
        ]

        # Execute command with opts
        result = runner.invoke(cli_frontend, opts)

        # Command stop on success exit code
        assert result.exit_code == 0

        # Validate return log output
        assert caplog.record_tuples == [
            (
                'boussole',
                20,
                "Project directory structure and configuration file have been created."
            ),
            (
                'boussole',
                20,
                "Now you should start to create some SASS sources into '{}', then compile them using:".format(sourcedir)
            ),
            (
                'boussole',
                20,
                '    boussole compile --config={}'.format(config_filepath)
            ),
        ]

        # Ensure dir and file has been created
        assert os.path.exists(config_filepath) == True
        assert os.path.exists(sourcedir) == True
        assert os.path.exists(targetdir) == True

        # Validate created configuration file
        with open(config_filepath, "rb") as fp:
            assert json.load(fp) == {
                'SOURCES_PATH': 'scss',
                'TARGET_PATH': 'css',
                "LIBRARY_PATHS": [],
                "OUTPUT_STYLES": "nested",
                "SOURCE_COMMENTS": False,
                "EXCLUDES": []
            }


def test_002(settings, caplog):
    """cli.startproject: Error from given arguments (multiple identical
       paths)"""
    runner = CliRunner()

    # Temporary isolated current dir
    with runner.isolated_filesystem():
        test_cwd = os.getcwd()

        sourcedir = os.path.join(test_cwd, "css")
        targetdir = os.path.join(test_cwd, "css")
        config_filepath = os.path.join(test_cwd, "settings.json")

        opts = [
            'startproject',
            '--basedir={}'.format(test_cwd),
            '--config={}'.format("settings.json"),
            '--sourcedir={}'.format("css"),
            '--targetdir={}'.format("css"),
        ]

        # Execute command with opts
        result = runner.invoke(cli_frontend, opts)

        # Command stop on success exit code
        assert result.exit_code == 1

        # Validate return log output
        assert caplog.record_tuples == [
            (
                'boussole',
                50,
                'Multiple occurences finded for path: {}'.format(sourcedir)
            )
        ]



# -*- coding: utf-8 -*-
import os
import pytest

def test_001(settings, inspector):
    """inspector.ScssInspector: Sub import dependencies"""
    sourcepath = os.path.join(settings.sample_path, 'main_with_subimports.scss')
    sources = [
        sourcepath,
    ]

    inspector.inspect(*sources)

    assert list(inspector.children(sourcepath)) == [
        os.path.join(settings.sample_path, "_empty.scss"),
        os.path.join(settings.sample_path, "_vendor.scss"),
        os.path.join(settings.sample_path, "components/_webfont.scss"),
        os.path.join(settings.sample_path, "main_basic.scss"),
        os.path.join(settings.sample_path, "components/_webfont_icons.scss"),
    ]

    assert list(inspector.parents(sourcepath))  == []



# -*- coding: utf-8 -*-
import os
import pytest

from boussole.exceptions import FinderException

def test_allowed_001(settings, finder):
    """finder.ScssFinder: Allowed simple filename"""
    allowed = finder.is_allowed("foo.scss", excludes=['*.css'])
    assert allowed == True


def test_allowed_002(settings, finder):
    """finder.ScssFinder: Allowed relative filepath"""
    allowed = finder.is_allowed("pika/foo.scss", excludes=['bar/*.scss'])
    assert allowed == True


def test_allowed_003(settings, finder):
    """finder.ScssFinder: Allowed relative filepath"""
    allowed = finder.is_allowed("pika/bar/foo.scss", excludes=['bar/*.scss'])
    assert allowed == True


def test_allowed_004(settings, finder):
    """finder.ScssFinder: Not allowed relative filepath"""
    allowed = finder.is_allowed("pika/bar/foo.scss", excludes=['foo.scss'])
    assert allowed == True


def test_allowed_005(settings, finder):
    """finder.ScssFinder: Allowed relative filepath with many patterns"""
    allowed = finder.is_allowed("pika/bar/foo.scss", excludes=[
        'foo.scss',
        'bar/*.scss',
        '*.css',
    ])
    assert allowed == True


def test_notallowed_100(settings, finder):
    """finder.ScssFinder: Not allowed simple filename"""
    allowed = finder.is_allowed("foo.scss", excludes=['*.scss'])
    assert allowed == False


def test_notallowed_101(settings, finder):
    """finder.ScssFinder: Not allowed simple filename"""
    allowed = finder.is_allowed("foo.scss", excludes=['foo.scss'])
    assert allowed == False


def test_notallowed_102(settings, finder):
    """finder.ScssFinder: Not allowed relative filepath"""
    allowed = finder.is_allowed("pika/foo.scss", excludes=['*.scss'])
    assert allowed == False


def test_notallowed_103(settings, finder):
    """finder.ScssFinder: Not allowed simple filename"""
    allowed = finder.is_allowed("foo.scss", excludes=['*.css', '*.scss'])
    assert allowed == False


def test_notallowed_104(settings, finder):
    """finder.ScssFinder: Not allowed relative filepath"""
    allowed = finder.is_allowed("pika/bar/foo.scss", excludes=['pika/*/*.scss'])
    assert allowed == False


def test_notallowed_105(settings, finder):
    """finder.ScssFinder: Not allowed relative filepath"""
    allowed = finder.is_allowed("pika/bar/plop/foo.scss", excludes=['pika/*/*.scss'])
    assert allowed == False


def test_notallowed_106(settings, finder):
    """finder.ScssFinder: Not allowed relative filepath, matching one of patterns"""
    allowed = finder.is_allowed("pika/bar/foo.scss", excludes=[
        'foo.scss',
        'bar/*.scss',
        'pika/*/*.scss', # bim
        '*.css',
    ])
    assert allowed == False


def test_notallowed_107(settings, finder):
    """finder.ScssFinder: Not allowed relative filepath, matching one of patterns"""
    allowed = finder.is_allowed("pika/bar/foo.scss", excludes=[
        'foo.scss',
        'bar/*.scss',
        '*.scss', # bam
        '*.css',
    ])
    assert allowed == False


def test_allowed_exception_201(settings, finder):
    """finder.ScssFinder: Absolute path raise an exception"""
    with pytest.raises(FinderException):
        allowed = finder.is_allowed("/foo.scss", excludes=[])



# -*- coding: utf-8 -*-
import os
import copy
import pytest

from boussole.exceptions import SettingsInvalidError
from boussole.conf.post_processor import SettingsPostProcessor


def test_001_success(settings):
    """conf.post_processor.SettingsPostProcessor: Validate required value on dummy value"""
    processor = SettingsPostProcessor()

    result = processor._validate_required({}, "DUMMY_NAME", "foo")
    assert result == "foo"


def test_002_fail(settings):
    """conf.post_processor.SettingsPostProcessor: Validate existing file on empty value"""
    processor = SettingsPostProcessor()

    with pytest.raises(SettingsInvalidError):
        processor._validate_required({}, "DUMMY_NAME", "")


def test_003_fail(settings):
    """conf.post_processor.SettingsPostProcessor: Validate existing file on empty value"""
    processor = SettingsPostProcessor()

    with pytest.raises(SettingsInvalidError):
        processor._validate_required({}, "DUMMY_NAME", None)



# -*- coding: utf-8 -*-
import os
import pytest

from boussole.exceptions import CircularImport

def test_self(settings, inspector):
    """inspector.ScssInspector: Ensure self import is detected"""
    sourcepath_0 = os.path.join(settings.sample_path, 'main_circular_0.scss')

    sources = [
        os.path.join(settings.sample_path, 'main_basic.scss'),
        sourcepath_0,
    ]

    inspector.inspect(*sources)

    with pytest.raises(CircularImport):
        children = list(inspector.children(sourcepath_0))

    with pytest.raises(CircularImport):
        parents = list(inspector.parents(sourcepath_0))

def test_direct(settings, inspector):
    """inspector.ScssInspector: Ensure direct circular imports are detected"""
    sourcepath_1 = os.path.join(settings.sample_path, 'main_circular_1.scss')
    sourcepath_2 = os.path.join(settings.sample_path, 'main_circular_2.scss')

    sources = [
        os.path.join(settings.sample_path, 'main_basic.scss'),
        os.path.join(settings.sample_path, 'main_with_subimports.scss'),
        sourcepath_1,
        sourcepath_2,
    ]

    inspector.inspect(*sources)

    with pytest.raises(CircularImport):
        children = list(inspector.children(sourcepath_1))

    with pytest.raises(CircularImport):
        parents = list(inspector.parents(sourcepath_1))

    with pytest.raises(CircularImport):
        children = list(inspector.children(sourcepath_2))

    with pytest.raises(CircularImport):
        parents = list(inspector.parents(sourcepath_2))


def test_indirect(settings, inspector):
    """inspector.ScssInspector: Ensure indirect circular imports are detected"""
    sourcepath_3 = os.path.join(settings.sample_path, 'main_circular_3.scss')
    sourcepath_4 = os.path.join(settings.sample_path, 'main_circular_4.scss')
    sourcepath_bridge = os.path.join(settings.sample_path, 'main_circular_bridge.scss')
    sourcepath_5 = os.path.join(settings.sample_path, 'main_circular_5.scss')

    sources = [
        os.path.join(settings.sample_path, 'main_basic.scss'),
        os.path.join(settings.sample_path, 'main_with_subimports.scss'),
        sourcepath_3,
        sourcepath_4,
        sourcepath_bridge,
        sourcepath_5,
    ]

    inspector.inspect(*sources)

    # Case 3
    with pytest.raises(CircularImport):
        children = list(inspector.children(sourcepath_3))

    with pytest.raises(CircularImport):
        parents = list(inspector.parents(sourcepath_3))

    # Case 4
    with pytest.raises(CircularImport):
        children = list(inspector.children(sourcepath_4))

    with pytest.raises(CircularImport):
        parents = list(inspector.parents(sourcepath_4))

    # Bridge
    with pytest.raises(CircularImport):
        children = list(inspector.children(sourcepath_bridge))

    with pytest.raises(CircularImport):
        parents = list(inspector.parents(sourcepath_bridge))


# NOTE: Disabled until i find a way to detected circular imports not
# directly involved by the source"""
#def test_004_subimport(settings, inspector):
    #"""inspector.ScssInspector: Ensure sub circular imports are detected"""
    #sourcepath_3 = os.path.join(settings.sample_path, 'main_circular_3.scss')
    #sourcepath_4 = os.path.join(settings.sample_path, 'main_circular_4.scss')
    #sourcepath_bridge = os.path.join(settings.sample_path, 'main_circular_bridge.scss')
    #sourcepath_5 = os.path.join(settings.sample_path, 'main_circular_5.scss')

    #sources = [
        #os.path.join(settings.sample_path, 'main_basic.scss'),
        #os.path.join(settings.sample_path, 'main_with_subimports.scss'),
        #sourcepath_3,
        #sourcepath_4,
        #sourcepath_bridge,
        #sourcepath_5,
    #]

    #inspector.inspect(*sources)

    ## Case 5
    #with pytest.raises(CircularImport):
        #children = list(inspector.children(sourcepath_5))

    #with pytest.raises(CircularImport):
        #parents = list(inspector.parents(sourcepath_5))



# -*- coding: utf-8 -*-
import os
import copy
import pytest

from boussole.exceptions import SettingsInvalidError
from boussole.conf.post_processor import SettingsPostProcessor


def test_001_success(settings, temp_builds_dir):
    """conf.post_processor.SettingsPostProcessor: Validate existing file path"""
    basedir = temp_builds_dir.join('postprocessor_validate_path_001')
    os.makedirs(basedir.strpath)

    processor = SettingsPostProcessor()

    foo = basedir.join('foo.txt')
    foo.write("Hello world!")

    result = processor._validate_path({}, "DUMMY_NAME", foo.strpath)

    assert result == foo.strpath


def test_002_exception(settings, temp_builds_dir):
    """conf.post_processor.SettingsPostProcessor: Validate not existing file path"""
    basedir = temp_builds_dir.join('postprocessor_validate_path_002')
    os.makedirs(basedir.strpath)

    processor = SettingsPostProcessor()

    foo = basedir.join('foo.txt')

    with pytest.raises(SettingsInvalidError):
        result = processor._validate_path({}, "DUMMY_NAME", foo.strpath)


def test_003_list_success(settings, temp_builds_dir):
    """conf.post_processor.SettingsPostProcessor: Validate existing file paths"""
    basedir = temp_builds_dir.join('postprocessor_validate_path_003')
    os.makedirs(basedir.strpath)

    processor = SettingsPostProcessor()

    foo = basedir.join('foo.txt')
    foo.write("Hello world!")

    bar = basedir.join('bar.txt')
    bar.write("Hello plop!")

    result = processor._validate_paths({}, "DUMMY_NAME", (
        foo.strpath,
        bar.strpath,
    ))

    assert result == [
        foo.strpath,
        bar.strpath,
    ]


def test_004_list_exception(settings, temp_builds_dir):
    """conf.post_processor.SettingsPostProcessor: Validate existing file paths"""
    basedir = temp_builds_dir.join('postprocessor_validate_path_004')
    os.makedirs(basedir.strpath)

    processor = SettingsPostProcessor()

    foo = basedir.join('foo.txt')
    foo.write("Hello world!")

    bar = basedir.join('bar.txt')
    bar.write("Hello plop!")

    with pytest.raises(SettingsInvalidError):
        result = processor._validate_paths({}, "DUMMY_NAME", (
            foo.strpath,
            "meh",
            bar.strpath,
        ))



# -*- coding: utf-8 -*-
import os
import json

import pytest

import click
from click.testing import CliRunner

from boussole.cli.console_script import cli_frontend


def test_verbosity_001(settings, caplog):
    """cli.compile: Testing default verbosity (aka INFO level) on setting
       error"""
    runner = CliRunner()

    # Temporary isolated current dir
    with runner.isolated_filesystem():
        test_cwd = os.getcwd()

        # Default verbosity
        result = runner.invoke(cli_frontend, ['compile'])
        assert caplog.record_tuples == [
            (
                'boussole',
                20,
                'Building project'
            ),
            (
                'boussole',
                50,
                'Unable to find settings file: {}/settings.json'.format(test_cwd)
            )
        ]
        assert 'Aborted!' in result.output
        assert result.exit_code == 1


def test_verbosity_002(settings, caplog):
    """cli.compile: Testing silent on setting error"""
    runner = CliRunner()

    # Temporary isolated current dir
    with runner.isolated_filesystem():
        test_cwd = os.getcwd()

        # Silent
        result = runner.invoke(cli_frontend, ['-v 0', 'compile'])

        error_msg = 'Unable to find settings file: {}/settings.json'.format(
            test_cwd
        )

        assert caplog.record_tuples == [
            (
                'boussole',
                50,
                error_msg
            )
        ]
        assert result.exit_code == 1

        # Totally silent output excepted the one from click.Abort()
        assert error_msg not in result.output
        assert 'Aborted!' in result.output

def test_verbosity_003(settings, caplog):
    """cli.compile: Testing debug level verbosity on setting error"""
    runner = CliRunner()

    # Temporary isolated current dir
    with runner.isolated_filesystem():
        test_cwd = os.getcwd()

        # Silent
        result = runner.invoke(cli_frontend, ['-v 5', 'compile'])

        error_msg = 'Unable to find settings file: {}/settings.json'.format(
            test_cwd
        )

        assert caplog.record_tuples == [
            (
                'boussole',
                20,
                'Building project'
            ),
            (
                'boussole',
                50,
                error_msg
            )
        ]
        assert error_msg in result.output
        assert 'Aborted!' in result.output
        assert result.exit_code == 1


def test_verbosity_004(settings, caplog):
    """cli.compile: Testing debug level verbosity on some file to
       compile"""
    runner = CliRunner()

    # Temporary isolated current dir
    with runner.isolated_filesystem():
        test_cwd = os.getcwd()

        # Need a file to compile, as setting error is not really verbose
        # Write a minimal config file
        with open('settings.json', 'w') as f:
            f.write(json.dumps({
                'SOURCES_PATH': '.',
                'TARGET_PATH': './css',
                'OUTPUT_STYLES': 'compact',
            }, indent=4))
        # Create needed dirs
        os.makedirs(os.path.join(test_cwd, "css"))

        # Write a minimal main SASS source
        source = "\n".join((
            """/* Main sample */""",
            """#content{""",
            """    color: red;""",
        ))
        with open('main.scss', 'w') as f:
            f.write(source)

        # Silent
        result = runner.invoke(cli_frontend, ['-v 5', 'compile'])

        error_msg = ("Error: Invalid CSS after \"    color: red;\": expected \"}\", "
                    "was \"\"\n        on line 3 of main.scss\n>>     "
                    "color: red;\n   ---------------^\n")

        assert result.exit_code == 1
        assert caplog.record_tuples == [
            ('boussole', 20, 'Building project'),
            ('boussole', 10, 'Project sources directory: {}'.format(test_cwd)),
            ('boussole', 10, 'Project destination directory: {}/css'.format(test_cwd)),
            ('boussole', 10, 'Exclude patterns: []'),
            ('boussole', 10, 'Compile: {}/main.scss'.format(test_cwd)),
            ('boussole', 40, error_msg)
        ]
        assert error_msg in result.output
        assert 'Aborted!' in result.output


def test_fail_001(settings, caplog):
    """cli.compile: Testing basic compile fail on default config filename
       (does not exists)"""
    runner = CliRunner()

    # Temporary isolated current dir
    with runner.isolated_filesystem():
        test_cwd = os.getcwd()

        result = runner.invoke(cli_frontend, ['compile'])

        assert result.exit_code == 1
        assert 'Aborted!' in result.output


def test_fail_002(settings):
    """cli.compile: Testing basic compile fail on given path directory (not a
       filename) as config file"""
    runner = CliRunner()

    # Temporary isolated current dir
    with runner.isolated_filesystem():
        test_cwd = os.getcwd()

        config_arg = '--config={}'.format(test_cwd)

        result = runner.invoke(cli_frontend, ['compile', config_arg])

        assert 'Aborted!' in result.output
        assert result.exit_code == 1


def test_fail_003(settings, caplog):
    """cli.compile: Testing basic compile fail on invalid config file (invalid
       JSON)"""
    runner = CliRunner()

    # Temporary isolated current dir
    with runner.isolated_filesystem():
        test_cwd = os.getcwd()

        with open('settings.json', 'w') as f:
            f.write('Invalid settings file content')

        result = runner.invoke(cli_frontend, ['compile'])

        msg = """No JSON object could be decoded from file: {}/settings.json"""
        assert result.exit_code == 1
        assert msg.format(test_cwd) in result.output
        assert 'Aborted!' in result.output


def test_fail_004(settings, caplog):
    """cli.compile: Testing exceptions management from sass compiler on
       invalid syntax"""
    runner = CliRunner()

    # Temporary isolated current dir
    with runner.isolated_filesystem():
        test_cwd = os.getcwd()

        # Write a minimal config file
        with open('settings.json', 'w') as f:
            f.write(json.dumps({
                'SOURCES_PATH': '.',
                'TARGET_PATH': './css',
                'OUTPUT_STYLES': 'compact',
            }, indent=4))

        # Create needed dirs
        os.makedirs(os.path.join(test_cwd, "css"))

        # Write a minimal main SASS source
        source = "\n".join((
            """/* Main sample */""",
            """#content{""",
            """    color: red;""",
        ))
        with open('main.scss', 'w') as f:
            f.write(source)

        # Invoke command action
        result = runner.invoke(cli_frontend, ['compile'])

        msg = """Invalid CSS after "    color: red;": expected "}", was """""

        assert result.exit_code == 1
        assert msg in result.output
        assert 'Aborted!' in result.output


def test_fail_005(settings, caplog):
    """cli.compile: Testing exceptions management from core API"""
    runner = CliRunner()

    # Temporary isolated current dir
    with runner.isolated_filesystem():
        test_cwd = os.getcwd()

        # Write a minimal config file
        with open('settings.json', 'w') as f:
            f.write(json.dumps({
                'SOURCES_PATH': '.',
                'TARGET_PATH': './css',
                'OUTPUT_STYLES': 'compact',
            }, indent=4))

        # Create needed dirs
        os.makedirs(os.path.join(test_cwd, "css"))

        # Write a minimal main SASS source
        source = "\n".join((
            """/* Main sample */""",
            """@import "mip";""",
        ))
        with open('main.scss', 'w') as f:
            f.write(source)

        # Invoke command action
        result = runner.invoke(cli_frontend, ['compile'])

        msg = """File to import not found or unreadable: mip"""

        assert result.exit_code == 1
        assert msg in result.output
        assert 'Aborted!' in result.output


def test_success_001(settings, caplog):
    """cli.compile: Testing compile success on basic config, a main SASS
       source and a partial source to ignore"""
    runner = CliRunner()

    # Temporary isolated current dir
    with runner.isolated_filesystem():
        test_cwd = os.getcwd()

        # Write a minimal config file
        with open('settings.json', 'w') as f:
            f.write(json.dumps({
                'SOURCES_PATH': '.',
                'TARGET_PATH': './css',
                'OUTPUT_STYLES': 'compact',
            }, indent=4))

        # Create needed dirs
        os.makedirs(os.path.join(test_cwd, "css"))

        # Write a minimal main SASS source
        source = "\n".join((
            """/* Main sample */""",
            """#content{""",
            """    color: red;""",
            """    &.wide{""",
            """        margin: 50px 15px;""",
            """    }""",
            """}""",
        ))
        with open('main.scss', 'w') as f:
            f.write(source)

        # Write a partial SASS source
        source = "\n".join((
            """/* Partial source to ignore */""",
            """.toignore-partial{""",
            """    color: gold !important;""",
            """}""",
        ))
        with open('_toignore.scss', 'w') as f:
            f.write(source)

        # Invoke command action
        result = runner.invoke(cli_frontend, ['compile'])

        # Attempted compiled CSS
        css_attempted = "\n".join((
            """/* Main sample */""",
            """#content { color: red; }""",
            "",
            """#content.wide { margin: 50px 15px; }""",
            "",
        ))
        with open(os.path.join(test_cwd, "css", "main.css"), 'rb') as f:
            css_compiled = f.read()

        #print result.exit_code
        #print result.output
        #print type(result)
        #print os.listdir(os.path.join(test_cwd, "css"))

        # Command success signal exit
        assert result.exit_code == 0
        # Source is correctly compiled
        assert css_compiled == css_attempted
        # Partial source to ignore is ignored
        assert os.listdir(os.path.join(test_cwd, "css")) == ['main.css']



# -*- coding: utf-8 -*-
import os
import pytest


def test_001(settings, finder):
    """finder.ScssFinder: Get destination for simple filename"""
    assert finder.get_destination("foo.scss") == "foo.css"


def test_002(settings, finder):
    """finder.ScssFinder: Get destination for simple filename"""
    assert finder.get_destination("foo.bar.scss") == "foo.bar.css"


def test_003(settings, finder):
    """finder.ScssFinder: Get destination for relative filepath"""
    assert finder.get_destination("mip/foo.scss") == "mip/foo.css"


def test_004(settings, finder):
    """finder.ScssFinder: Get absolute destination for filepath"""
    result = finder.get_destination("foo.scss", targetdir="/home")
    assert result == "/home/foo.css"


def test_005(settings, finder):
    """finder.ScssFinder: Get absolute destination for filepath"""
    result = finder.get_destination("mip/foo.scss", targetdir="/home")
    assert result == "/home/mip/foo.css"


def test_006(settings, finder):
    """finder.ScssFinder: Wrong destination troubled with absolute filepath"""
    result = finder.get_destination("/etc/mip/foo.scss", targetdir="/home")
    assert result == "/etc/mip/foo.css"



# -*- coding: utf-8 -*-
import os
import json
import pytest


def test_001(projectstarter, temp_builds_dir):
    """project.init_project: Testing on basic values"""
    basedir = temp_builds_dir.join('projectstarter_commit_001').strpath
    os.makedirs(basedir)

    opts = (
        "scss",
        "css",
        os.path.join(basedir, "settings.json"),
        os.path.join(basedir, "scss"),
        os.path.join(basedir, "css"),
    )

    projectstarter.commit(*opts)

    assert os.path.exists(os.path.join(basedir, "settings.json")) == True
    assert os.path.exists(os.path.join(basedir, "scss")) == True
    assert os.path.exists(os.path.join(basedir, "css")) == True

    with open(os.path.join(basedir, "settings.json"), "rb") as fp:
        assert json.load(fp) == {
            'SOURCES_PATH': 'scss',
            'TARGET_PATH': 'css',
            "LIBRARY_PATHS": [],
            "OUTPUT_STYLES": "nested",
            "SOURCE_COMMENTS": False,
            "EXCLUDES": []
        }
